;*****************************************************************************
;
;	CC400DB		Demonstrator Board Software
;
;	File : CC400DB20.ASM
;
;	Written for ATtiny12 (1k FLASH, 64 bytes EEPROM, no SRAM)
;	AVR runs off internal 1.2 MHz oscillator
;
;	Data decision is done using an oversampling-based algorithm. Each
;	data byte is sampled 8 times, and a majority vote decides whether it
;	is a 0 or a 1.
;
;	Author : Karl H. Torvmark, Chipcon FAE
;
;	For technical questions, contact support@chipcon.com
;
;	Version :	2.0 2001-10-24
;	Totally rewritten from 1.0 to 2.0. Controller changed from AT90LS2343
;	to ATtiny12. Took advantage of the speed increase of the controller
;	to implement oversampling. Software rewritten to function without
;	SRAM.
;
;*****************************************************************************



;*****************************************************************************
;
; DESCRIPTION
;
; General
;
;	This software demonstrates data transmission and reception using the
;	CC400 RF transceiver chip from Chipcon and an AVR microcontroller
;	from Atmel.
;
;	The software is implemented as a state machine which handles user
;	input and polling for incoming RF data. 
;
; Hardware
;
;	The software is written for an ATtiny12 controller. This represents
;	the smallest MCU which is feasable for use in an application. It
;	provides 1K of program memory, 64 bytes of non-volatile EEPROM
;	data memory, and no SRAM memory (all data must be kept in the
;	32 internal registers of the AVR). The lack of SRAM also means that
;	the application must run using a 3-level hardware stack. The
;	application has been written with these restrictions in mind. The
;	practical implications is that all code is run from interrupt handlers,
;	no subroutines should be called from the main program when interrupts
;	are enabled. Nested interrups are not supported. Some "functions" are
;	not functions (the CC400 configuration routines), but are called using
;	rjmp. The interrupt handlers use almost all registers (see resources
;	for more info), but do not disturb SREG.
;
;	If a controller with SRAM is used, incoming RF data can be stored in
;	SRAM as it arrives, and be decoded later. Also, the function calls can
;	be cleaned up, and more registers can be freed up for use by the
;	main program.
;
;	Since the CC400DB is powered by a battery, the system must function
;	with a varying battery voltage. The demonstration board uses a
;	lithium battery. The practical terminal voltage may vary from 3.6 to
;	2.7 volts. The MCU's internal clock must be stable enough over this
;	voltage supply range for reliable data communication to be possible.
;	This was the reason for changing controllers. The RC oscillator of the
;	AT90LS2343 varies from under 300kHz to over 500kHz over the 2.7-3.6V supply
;	range, and this makes reliable data communication difficult. The RC oscillator
;	of the ATtiny12 is much improved, and the frequency will lie between
;	1.11 and 1.18 MHz for the specified voltage range (this also includes
;	variations with temperature). For applications where the MCU is powered
;	by a battery, the oscillator must provide this level of stability.
;	Good RC or ceramic oscillators are usable, a quartz crystal solution
;	is the optimum solution. The CC400 requires a clock source with a
;	stability of +/- 50 ppm or better, if the MCU has more than 8 pins,
;	a good solution will be to run the MCU on a 4-13MHz crystal, and
;	let the CC400 run on the same clock.	
;
;	The 1.1MHz clock enables us to implement an oversampling-based
;	data decision algorithm, which is less sensitive to noise and
;	duty-cycle variations than an algorithm based on edge-detection.
;	This results in a better data error rate, especially when the receiver
;	is near the sensitivity limit.
;
; Power consumption
;
;	The characterisation data for the ATtiny12 MCU (taken from the newest
;	data sheet from Atmel), gives the following power consumption for the
;	controller running on the internal oscillator at 3.3V :
;		0.7-0.8mA in active mode
; 		0.3mA in idle mode
; 		<1uA in sleep with watchdog disabled
; 		5-10uA in sleep with watchdog enabled
;
;	The CC400 data sheet gives the following power consumption:
; 		RX 		: typ. 18mA
; 		TX (0 dBm)	: typ. 29mA
; 		PD (osc. off) 	: typ. 0.2uA
;
;	Measured values: (@ 3.3V)
;		Avg. in polling : 1.3mA
;		TX		: 30mA
;		Power down	: <1uA
;
;	The board has a power consumption of less than 1 uA in power down,
;	giving long battery life. The software is written so that the 
;	transceiver spends as much time in power down as possible. In polling
;	mode, the MCU wakes up every 200ms, but only wakes up the CC400
;	each fourth time (approx. once per second). The software waits for
;	a positive edge (start of a '1' in Manchester code), and oversamples
;	two bits. If these bits are good Manchester coded 1's (part of a
;	preamble), the software starts looking for a start-of-frame (SOF).
;	If not, the MCU shuts down the transceiver and goes back to idle
;	mode to conserve battery power.
;
;	When the MCU has woken up 256 times since the last button press
;	or received data, it goes into sleep mode, and must be waked using
;	the ON button.
;
; Performance:
;	The range has been measured to be in excess of 150m (line-of-sight)
;	The software can receive data with a bitrate within +/- 10% of
;	its own.
;
; Resource usage
;
;	The program uses 476 words of program memory, while stored configuration
;	constants for the various states uses 29 words. This leaves very little
;	memory for further development. A larger microcontroller is recommended
;	for further application development. An application using any form of
;	protocol except the bare minimum should be implemented on a 
;	microcontroller with SRAM.
;
;	This application uses the RESET, OVF0, INT0 and PCINT interrupts.
;	The RESET interrupt initialises the controller, and is called
;	on power-up or after a watchdog timeout. The OVF0 interrupt occurs
;	when the 8-bit timer overflows, and is used for a wide variety of
;	timing functions dependent on which state the program is in.
;	INT0 occurs when the state of the INT0 pin is changed, it is
;	used to wake up the MCU from sleep, and to synchronise with the
;	first rising edge of incoming data when receiving.
;	The PCINT interrupt occurs when any I/O pin is toggled, and is
;	used to monitor the TX button.
;
;	The ERDY EEPROM ready interrupt and the ACI analog comparator
;	interrupt are not used.
;
; Implementation:
;	All program functions are interrupt-based; the main loop of the
;	program is merely an infinite loop containing a sleep instruction.
;
;	The software is implemented as a state-machine. The program starts
;	in the POLLING state, and enters other states according to
;	inputs.
;
;	When the MCU is reset, either by the power-up reset function or
;	by the watchdog timer, the program initialises the MCU and
;	enters the POLLING state.
;
;	In the POLLING state, the OVF0 interrupt is active. Each 200ms,
;	the MCU wakes up. On each 4th interrupt, the program enters
;	the PREAMBLE state. If the TX button is pressed, the PCI interrupt
;	occurs, and the program enters the TX state. If the OVF0 interrupt
;	has occurred 256 times without valid data being received or the
;	TX button been pressed, the program enters the PD state.
;	In the POLLING state, the CC400 is in power-down.
;
;	In the PD state, only the INT0 interrupt is enabled. The MCU and
;	the CC400 is in power-down. When the TX button is pressed, the
;	INT0 interrupt occurs, and the program enters the POLLING state
;	again.
;
;	In the PREAMBLE state, the CC400 is put first into RX_PRECHARGE,
;	then into RX mode. When this is finished, the INT0 interrupt is
;	enabled. An INT0 interrupt occurs on the first positive transition.
;	The program then sets up the OVF0 interrupt so that 8 samples
;	will be taken in each bit period. When the program has sampled
;	two bits, the results are evalutated to find out if two valid
;	Manchester coded 1's have been found. If not, the program
;	enters polling mode again. If we have found 2 1's, this indicates
;	that a preample is received, and the program enters the SOF_SEARCH
;	state. The PREAMBLE state is implemented to save battery power. If
;	no valid data is detected (presumably the most common case), the
;	CC400 spends a minimum amount of time in RX mode, which conserved
;	battery lifetime.
;
;	In the SOF_SEARCH state, the program reads the incoming data
;	stream, waiting for the SOF (Start Of Frame) to appear. The
;	SOF is implemented as two consequtive 0's. If the SOF is found,
;	the program enters the RX state. If no SOF is found within a
;	time-out period, or if too many bit errors are detected, the
;	program aborts and returns to the POLLING state.
;
;	In the RX state, the SOF has been received, and the program is
;	receiving data fields. The ID field is checked, if this is not
;	0 (broadcast), the program returns to the POLLING state. The TYPE
;	field is checked, if this is set to REQUEST, a flag is set
;	indicating that an acknowledge message should be sent.
;	If the TYPE field is not a valid value, the program returns to
;	the POLLING state. If both the ID and TYPE fields contain correct
;	values, the 3 DATA fields are read into a buffer. When all the
;	data is received, the program enters the DISPLAY state.
;
;	In the DISPLAY state, the OVF0 interrupt is active. Each interrupt,
;	a new bit from the data buffer is displayed on the LED (MSB first).
;	When all the bits have been displayed, the program sets the ACK bit
;	set in the RX state. If an acknowledge is to be sent, the
;	program enters the TX state, otherwise the program returns to POLLING.
;
;	In the TX state, the program transmits data. The CC400 is put into
;	TX mode. The OVF0 interrupt occurs two times each bit period, transmitting
;	the first and second chips of the Manchester coded signal to the
;	RF transceiver. First, a preamble is transmitted. Then a SOF and finally
;	the data fields. When the data has been sent, the program returns to the
;	POLLING state.




.include "tn12def.inc"


;*****************************************************************************
; Configuration data
;
; Stored in EEPROM
;
; Updated with recommended settings from SmartRF Studio 3.0
;
;*****************************************************************************

.eseg                                                                       
 
TX:          ; 8x16bit  (Note: only frame 007 is different from RX pre-charge)
    .db 0x08, 0x2A ; Adr:000    Data:082A
    .db 0x33, 0x0B ; Adr:001    Data:130B
    .db 0x40, 0x41 ; Adr:002    Data:0041
    .db 0x7F, 0x9A ; Adr:003    Data:1F9A
    .db 0x8A, 0x29 ; Adr:004    Data:0A29
    .db 0xBA, 0x02 ; Adr:005    Data:1A02
    .db 0xCB, 0x6C ; Adr:006    Data:0B6C
    .db 0xE0, 0x40 ; Adr:007    Data:0040

RX:          ; 1x16bit
    .db 0xE0, 0x40 ; Adr:007   Data:0040 

RX_PC:       ; (Pre-charge mode):  8x16bit    RX pre-charge
    .db 0x00, 0x2A ; Adr:000   Data:002A
    .db 0x33, 0x0B ; Adr:001   Data:230B
    .db 0x40, 0x41 ; Adr:002   Data:0041
    .db 0x7E, 0xE1 ; Adr:003   Data:1EE1
    .db 0x8A, 0x00 ; Adr:004   Data:0A00
    .db 0xBA, 0x02 ; Adr:005   Data:1A02
    .db 0xCB, 0x6C ; Adr:006   Data:0B6C
    .db 0xE2, 0x48 ; Adr:007   Data:0248

PD:          ; Power down, osc. off, 2x16bit 
    .db 0x18, 0x2A ; Adr:000   Data:182A 
    .db 0x40, 0x51 ; Adr:002   Data:0051 
 

.cseg
     
;*****************************************************************************
; Variables
;*****************************************************************************

.def	temp		= r29		; General temporary register
.def	ee_counter	= r17		; Counter during configuration
.def	ee_pointer	= r18		; EEPROM configuration pointer
.def	ee_buffer	= r19		; EEPROM configuration buffer
.def	sreg_save	= r1		; Saves SREG during interrupts
.def	state		= r16		; Contains state info
.def	poll_count	= r20		; Number of polling attempts
.def	preamble_count	= r21		; Number of bits checked
; The following two variables share the same physical register, as
; they are active in different states
.def	preamble_OK	= r22		; Number of preamble bits found
.def	error_allow	= r22		; Number of errors allowed during SOF
.def	An0		= r25		; Current A value
.def	An1		= r28		; A value one sample old
.def	An2		= r3		; A value two samples old
.def	ShiftReg	= r4		; Shift register for oversampling
.def	Sample		= r24		; Number of current sample
.def	BitCountLow	= r26		; Low register of bit counter
.def	BitCountHigh	= r27		; High register of bit counter
.def	data_buffer	= r13		; Used for buffering data in RX
.def	data1		= r5		; Storage of data1
.def	data2		= r6		; Storage of data2
.def	data3		= r7		; Storage of data3
.def	id		= r8		; Storage of id
.def	type		= r9		; Storage of type
.def	ZeroReg		= r10		; Always set to 0
.def	OneReg		= r11		; Always set to 1
.def	LastBit		= r12		; Stores the previous bit

.def	flag_reg	= r23		; Register containing various flags
; b0 : First Bit Stored (valid in preamble)
; b1 : Resync enable (valid when oversampling)
; b2 : Send ACK (valid in RX)
; b3 : Preamble (valid in TX)
; b4 : SOF (valid in TX)
; b5 : LastBit (valid in TX)

; Bit definitions for flag_reg
.equ	FIRST_BIT_STORED	= 0
.equ	RESYNC_ENABLE		= 1
.equ	SEND_ACK		= 2
.equ	PREAMBLE		= 3
.equ	SOF			= 4
.equ	LAST_BIT		= 5

; Note: R0, R30 and R31 are used for special purposes, and must be reserved


;*****************************************************************************
; Constants
;*****************************************************************************

; I/O pin constants
;
; ATtiny12 pin-out
;				 _______ _______
;			      	|       V       |
;		#RESET	     ---| 1           8 |----	VCC
; 	STROBE	PB3	     ---| 2 ATtiny12  7 |----	PB2/SCK	       TX/CLOCK
;	PDATA	PB4	     ---| 3           6 |----	PB1/MISO/INT0  DIO/ON
;		GND	     ---| 4           5 |----	PB0/MOSI       LED
;				|_______________|


.equ	LED			= 0
.equ	DIO			= 1
.equ	CLOCK			= 2
.equ	STROBE			= 3
.equ	PDATA			= 4

.equ	ON_BUTTON		= 1
.equ	TX_BUTTON		= 2

.equ	PORTB_POLLING		= (1<<LED | 0<<PDATA | 1<<CLOCK | 0<<STROBE | 1 << DIO)
.equ	PORTB_PD		= (1<<LED | 0<<PDATA | 0<<CLOCK | 0<<STROBE | 1 << DIO)
.equ	PORTB_PREAMBLE		= (1<<LED | 0<<PDATA | 0<<CLOCK | 0<<STROBE | 1 << DIO)
.equ	PORTB_SOFSEARCH		= (1<<LED | 0<<PDATA | 0<<CLOCK | 0<<STROBE | 1 << DIO)
.equ	PORTB_DISPLAY		= (1<<LED | 0<<PDATA | 0<<CLOCK | 0<<STROBE | 1 << DIO)
.equ	PORTB_TX		= (1<<LED | 0<<PDATA | 0<<CLOCK | 0<<STROBE | 1 << DIO)
.equ	PORTB_RX		= (1<<LED | 0<<PDATA | 0<<CLOCK | 0<<STROBE | 1 << DIO)

.equ	DDRB_POLLING		= (1<<LED | 1<<PDATA | 0<<CLOCK | 1<<STROBE | 0 << DIO)
.equ	DDRB_PD			= (1<<LED | 1<<PDATA | 0<<CLOCK | 1<<STROBE | 0 << DIO)
.equ	DDRB_PREAMBLE		= (1<<LED | 1<<PDATA | 0<<CLOCK | 1<<STROBE | 0 << DIO)
.equ	DDRB_SOFSEARCH		= (1<<LED | 1<<PDATA | 0<<CLOCK | 1<<STROBE | 0 << DIO)
.equ	DDRB_DISPLAY		= (1<<LED | 1<<PDATA | 0<<CLOCK | 1<<STROBE | 0 << DIO)
.equ	DDRB_TX			= (1<<LED | 1<<PDATA | 0<<CLOCK | 1<<STROBE | 1 << DIO)
.equ	DDRB_RX			= (1<<LED | 1<<PDATA | 0<<CLOCK | 1<<STROBE | 0 << DIO)

.equ	GIMSK_POLLING		= 0x20
.equ	GIMSK_PD		= 0x40
.equ	GIMSK_PREAMBLE		= 0x00
.equ	GIMSK_SOFSEARCH		= 0x00
.equ	GIMSK_DISPLAY		= 0x00
.equ	GIMSK_TX		= 0x00
.equ	GIMSK_RX		= 0x00

.equ	ACSR_INIT		= 0x80

.equ	MCUCR_PD		= 0x30
.equ	MCUCR_POLLING		= 0x03
.equ	MCUCR_PREAMBLE		= 0x03
.equ	MCUCR_SOFSEARCH		= 0x03
.equ	MCUCR_DISPLAY		= 0x03
.equ	MCUCR_TX		= 0x03
.equ	MCUCR_RX		= 0x00



; Protocol constants

.equ	PREAMBLE_LENGTH		= 1800		; Approx. 1.5 second long preamble (1800 bits)
.equ	PACKET_REQUEST		= 0x80		; Request (send response)
.equ	PACKET_ACKNOWLEDGE	= 0x81		; Acknowledge (do not send response)
.equ	PACKET_UNKNOWN		= 0xF2		; Unknown packet
.equ	ID_BYTE			= 0x00		; ID : Broadcast

.equ	DATA1_VALUE		= 0b10101011	; Blinking sequence
.equ	DATA2_VALUE		= 0b10000001	; 2x morse 'V'
.equ	DATA3_VALUE		= 0b01010111	; -'victory'


; Timer constants for various states

.equ	PD_TIMER_PRESCALE	= 0x01		; Need a value here,
.equ	PD_TIMER_PRESET		= 0x01		; can use any

; 1200 bps, 1.2MHz clock frequency gives us 1000 instructions per bit
; with 8 times oversampling : 125 instructions/sample
; Some instructions overhead, counter value of 0x97 gives correct timing

.equ	RX_TIMER_PRESCALE	= 0x01		; Divide by 1
.equ	RX_TIMER_PRESET		= 0x97		; 104 instructions

; In polling mode we want one interrupt/sec, but the counter can't
; provide this directly. We use the longest period we can have,
; and count interrupts in software

.equ	POLLING_TIMER_PRESCALE	= 0x05		; Divide by 1024
.equ	POLLING_TIMER_PRESET	= 0x01		; 262144 instructions (218ms) 

; In preamble mode we want the same timing as in RX

.equ	PREAMBLE_TIMER_PRESCALE	= 0x01
.equ	PREAMBLE_TIMER_PRESET	= 0x97

; In SOF search mode we want the same timing as in RX also

.equ	SOFSEARCH_TIMER_PRESCALE= 0x01
.equ	SOFSEARCH_TIMER_PRESET	= 0x7D
				

; In TX mode we want interrupts at twice the baudrate
; 2400 interrupts/sec gives us 500 instructions/interrupt
.equ    TX_TIMER_PRESCALE	= 0x02		; Divide by 8
.equ	TX_TIMER_PRESET		= 0xC5 		; 62*8=496 instructions


; In display, we want interrupts to occur at a rate
; so that the LED blinking is readable, approx. 100ms per bit
; 10 interrupts/sec gives us 120000 instructions/interrupt
.equ	DISPLAY_TIMER_PRESCALE	= 0x05		; Divide by 1024
.equ	DISPLAY_TIMER_PRESET	= 0x7F		; 130048 instructions (114ms)


; States
.equ	STATE_POLLING		= 0x00
.equ	STATE_TX		= 0x01
.equ	STATE_RX		= 0x02
.equ	STATE_PD		= 0x03
.equ	STATE_PREAMBLE		= 0x04
.equ	STATE_SOFSEARCH		= 0x05
.equ	STATE_DISPLAY		= 0x06




; Oversampling constants

.equ	A_THRESHOLD		= 0x04		; Threshold for data decision

.equ	PREAMBLE_THRESHOLD	= 0x05		; Threshold for preamble
.equ	SOF_THRESHOLD		= 0x03		; Threshold for SOF
.equ	MAX_SOF_ERRORS		= 0x05		; Allow 5 errors before giving
						; up SOF search



;*****************************************************************************
; Macros
;*****************************************************************************

.macro 	LED_ON
	sbi	DDRB, LED
	cbi	PORTB, LED			; Turn on LED
.endmacro

.macro 	LED_OFF
	
	sbi	PORTB, LED			; Turn off LED
.endmacro


.cseg
.org 0x000
	rjmp 	RESET
.org INT0addr
	rjmp 	INT0_HANDLER
.org PCINTaddr
	rjmp 	PCINT
.org OVF0addr
	rjmp 	OVF0
.org ERDYaddr
	rjmp 	RESET				; Not used
.org ACIaddr
	rjmp 	RESET				; not used


RESET:

	; Disable analog comparator
	ldi	temp, ACSR_INIT
	out	ACSR, temp
	
	; Read osc. calibration value from flash, and update register
	ldi	ZL,low(CALIBRATION_BYTE<<1)+1
	ldi	ZH,high(CALIBRATION_BYTE<<1)
	lpm
	out	osccal,r0
	
	
	; Enable timer/counter
	ldi	temp, 0x02
	out	TIMSK, temp
	
	
	
	
INIT_VAR:	; Initialise variables


	clr	poll_count		; Clear polling counter
	clr	flag_reg		; Clear flags
	
	clr	ZeroReg			; Set up zero register
	
	ldi	temp,1
	mov	OneReg,temp		; Set up one register

	rcall	START_POLLING_STATE	; Default state : POLLING
	
	LED_ON				; Light LED
	
WAIT_FOR_ON:
	sbis	PINB, DIO		
	rjmp	WAIT_FOR_ON		; Do not go on until ON button is released
	
	LED_OFF
	
	; Start main program
	
	sei				; Enable interrupts
	rjmp	MAIN
	
	
	; Main program loop
MAIN:
	sleep				; In PD, enter sleep mode,
					; in other modes, enter idle mode
	
	rjmp	MAIN			; Proceed after each interrupt


;*****************************************************************************
;
; Resynchronisation routine
;
;*****************************************************************************

RESYNC:
	cp	An2,An1			; An2>=An1?
	brge	RESYNC_1		; If yes, jump
	
	cp	An1,An0			; An1<An0?
	brlo	RESYNC_2		; If yes, jump
	
	rjmp	NO_RESYNC		; No resync
	
RESYNC_1:
	cp	An1,An0			; An1>=An0?	
	brge	RESYNC_3		; If yes, jump
	
	rjmp	NO_RESYNC		; No resync
RESYNC_2:
	cpi	An1, A_THRESHOLD	; An1<A_THRESHOLD?
	brlo	R_ADD2			; If yes, jump
	
	rjmp	FINISH_RESYNC
	
RESYNC_3:
	cp	An2,An1			; An2=An1?
	breq	NO_RESYNC		; If yes, jump
	cp	An0,An1			; An0=An1?
	breq	NO_RESYNC		; If yes, jump
	
	cpi	An1,A_THRESHOLD		; An1>=A_THRESHOLD?
	brge	R_ADD2			; If yes, jump
	
	rjmp	FINISH_RESYNC
		

R_ADD2:
	inc	Sample
	
NO_RESYNC:
	inc	Sample	
	
FINISH_RESYNC:
	cbr	flag_reg, 1<<RESYNC_ENABLE ; No resync next bit

	rjmp	OVERSAMPLING_END


;*****************************************************************************
;
; Oversampling routine
;
;*****************************************************************************

OVERSAMPLING:
	ldi	temp, RX_TIMER_PRESET
	out	TCNT0, temp		; Reload timer as quickly as possible,
					; timing is the same for all states
					; which use OVERSAMPLING


	mov	An2, An1		; Store old values of A
	mov	An1, An0
	
	
	clc				; Carry = DIO
	sbic	PINB, DIO
	sec			
	
	rol	ShiftReg		; Shift new sample into shift register
	
	
	; Update value of A according to bits shifted in and out
	; A is equal to the number of 0's in the shift register
	
	brcs	A_ADJ1			; Check bit shifted out, if 1, don't update A
	dec	An0			; If 0, decrease A

A_ADJ1:

	bst	ShiftReg,0		; Check new sample
	brts	A_ADJ2			; If 1, don't update A
	
	inc	An0			; If 0, increase A
	
A_ADJ2:	
	
	ldi	temp, 0x10
	eor	ShiftReg,temp		; Invert new middle bit
	
	bst	ShiftReg,4		; Check bit inverted
	brtc	A_ADJ3			; If 0, jump
	
	dec	An0			; If 1, decrease A
	rjmp	A_ADJ4			; Finished
A_ADJ3:	
	inc	An0			; If 0, increase A
	
A_ADJ4:

	sbrc	flag_reg, RESYNC_ENABLE	; Need to resync?
	rjmp	RESYNC			; If yes, go to resync routine

	inc	Sample
	cpi	Sample,8		; Last sample in bit?
	brne	OVERSAMPLING_END	; No, we're done
	
	
	clr	Sample			; Finished with bit, reset sample counter
	
	
	; Handle end-of-bit processing for different states
	
	cpi	state,STATE_PREAMBLE
	brne	SKIP
	
	rjmp	OVERSAMPLING_PREAMBLE
	
SKIP:
	
	cpi	state,STATE_SOFSEARCH
	breq	OVERSAMPLING_SOFSEARCH
	
	cpi	state,STATE_RX
	breq	OVERSAMPLING_RX
	
	rjmp	OVERSAMPLING_END	
	
OVERSAMPLING_SOFSEARCH:

	; During SOF search, we resynchronise continually, and search
	; for two consecutive 0's, which indicate start of packet
	; If we do not find two consecutive 0's when we have received
	; the max number of bits defined for the preamble, we give up
	; and go back to polling.
	
	
	sbr	flag_reg, 1<<RESYNC_ENABLE	; Resynchronise next sample
	
	
	; Check if we should give up finding the SOF
	cpi	BitCountHigh, high(PREAMBLE_LENGTH)
	brlo	OS_CONT
	cpi	BitCountLow, low(PREAMBLE_LENGTH)
	brlo	OS_CONT
	
	; We have timed out
	
	rcall	START_POLLING_STATE	; Go back to polling
	rjmp	OVERSAMPLING_END		
		
	
OS_CONT:

	; No time-out	
	
	add	BitCountLow,OneReg	; Add 1 to bit counter, low byte
	adc	BitCountHigh,ZeroReg	; Update high byte as well
	
	cpi	An0, PREAMBLE_THRESHOLD	; Is bit an acceptable 1?
	brsh	OVERSAMPLING_END	; If yes, we are still receiving
					; a preamble, exit
					
	; Potential SOF
	

	cpi 	An0, SOF_THRESHOLD	; Is bit an acceptable SOF?
	brge	SOF_FAIL		; If not, exit
	
	ldi	temp, SOF_THRESHOLD
	cp	LastBit,temp		; Was the previous bit a SOF?
	brge	SOF_FAIL		; If not, exit
	
	rjmp	SOF_OK			; OK, handle it
	
SOF_FAIL:
	mov	LastBit,An0		; Save bit for next time
	subi	error_allow,0x01
	brne	OVERSAMPLING_END	; If OK, continue
	
	; Too many errors, go back to
	; polling
	rcall	START_POLLING_STATE
	rjmp	OVERSAMPLING_END
	
SOF_OK:
	; We have received a valid SOF

	
	rcall	START_RX_STATE		; Go into data reception
	
	rjmp	OVERSAMPLING_END			


OVERSAMPLING_END:
	out	SREG, SREG_SAVE
	reti
	
	
OVERSAMPLING_RX:

	; During RX, we receive data and process it as soon as a
	; byte is complete (since the tiny12 does not have SRAM,
	; data buffering is impossible)
	
	sbr	flag_reg, 1<<RESYNC_ENABLE	; Resynchronise next sample


	cpi	An0, A_THRESHOLD	; Carry=(A_THRESHOLD>An0)
	
OR_1:
	rol	data_buffer		; Shift in new bit from carry
	
	inc	BitCountLow		; Update counter
	
	mov	temp,BitCountLow
	andi	temp,0x07		; Have we received an entire byte?
	
	brne	OVERSAMPLING_END	; If not, exit
	
	; data_buffer contains a byte which must be processed
	
	cpi	BitCountLow,0x08	; Is this the ID byte?
	brne	OR_2			; If not, jump
	
	ldi	temp,ID_BYTE
	cp	data_buffer,temp	; ID byte set to public address?
	brne	FAIL			; If not, fail and return to polling
	rjmp	OVERSAMPLING_END	; Finished processing ID byte
	
OR_2:
	cpi	BitCountLow,0x10	; Is this the type byte?
	brne	OR_3			; If not, jump
	
	ldi	temp, PACKET_REQUEST
	cp	data_buffer,temp
	brne	OR_ACK			; If not a request, jump
	
	
	sbr	flag_reg, 1<<SEND_ACK	; Set flag to send ACK
	rjmp	OVERSAMPLING_END	; Finished processing type byte

OR_ACK:
	ldi	temp, PACKET_ACKNOWLEDGE
	cp	data_buffer,temp
	brne	FAIL			; If type unknown, fail and return to polling
	
	cbr	flag_reg, 1<<SEND_ACK	; Clear flag, do not send ACK
	rjmp	OVERSAMPLING_END	; Finished processing type byte
	
OR_3:
	cpi	BitCountLow,0x18	; Is this the first data byte?
	brne	OR_4			; If not, jump
	
	mov	data1, data_buffer	; Store data
	rjmp	OVERSAMPLING_END	; Finished processing data1
	
OR_4:
	cpi	BitCountLow,0x20	; Is this the second data byte?
	brne	OR_5			; If not, jump
	
	mov	data2, data_buffer	; Store data
	rjmp	OVERSAMPLING_END	; Finished processing data2
	
OR_5:
	cpi	BitCountLow,0x28	; Is this the third data byte?
	brne	DATA_END		; If not, jump
	
	mov	data3, data_buffer	; Store data
	
DATA_END:
	rcall	START_DISPLAY_STATE	; Enter display data state
	rjmp	OVERSAMPLING_END
	
FAIL:
	rcall	START_POLLING_STATE	; Go back to polling
	rjmp	OVERSAMPLING_END
	


	
OVERSAMPLING_PREAMBLE:

	; During preamble, we do not resyncronise, we just
	; read two bits. If they are both valid 1's, we
	; start resynchronising, else we decide no transmitter is active
	; This is a form of software squelch
	
		
	sbrc	flag_reg,FIRST_BIT_STORED
	rjmp	OP_SECOND_BIT	; If second bit, handle it
	
	; If first bit, set flag and exit
	
	sbr	flag_reg,(1<<FIRST_BIT_STORED)
	mov	LastBit,An0
	rjmp	OVERSAMPLING_END
	
	
	
OP_SECOND_BIT:
	ldi	temp, PREAMBLE_THRESHOLD
	cp	LastBit, temp
	brlo	OP_FAIL			; First bit below threshold
	
	cpi	An0, PREAMBLE_THRESHOLD
	brlo	OP_FAIL			; Second bit below threshold
	
OP_SUCCESS:
	; Both bits were valid 1's
	cbr	flag_reg,1<<FIRST_BIT_STORED
	
	rcall	START_SOFSEARCH 	; Start searching for SOF
	
	rjmp	OVERSAMPLING_END
	
OP_FAIL:
	; One of the two bits is not a valid 1
	cbr	flag_reg,1<<FIRST_BIT_STORED
	
	rcall	START_POLLING_STATE	; Go back to polling mode
	
	rjmp	OVERSAMPLING_END


	
;*****************************************************************************
;
; Timer overflow interrupt handler
;
;*****************************************************************************	
	
OVF0:
	
	wdr				; Strobe the watchdog, program is
					; running OK	
	
	in	SREG_SAVE, SREG		; Save SREG
					
			
	; Determine state
	
	mov	temp, state
	
	cpi	temp, STATE_POLLING	
	breq	OVF0_POLLING		; Handle polling state
	
	cpi	temp, STATE_TX
	breq	OVF0_TX			; Handle TX state
	
	cpi	temp, STATE_DISPLAY
	brne	HANDLE_REST		
	rjmp	OVF0_DISPLAY		; Handle display state
	
HANDLE_REST:
	rjmp	OVERSAMPLING		; Rest of the states are handled by
					; the oversampling routine
					
; Timer overflow handler for TX state
	
OVF0_TX:
	ldi	temp, TX_TIMER_PRESET
	out	TCNT0, temp		; Reload timer
	
	
	sbrs	flag_reg, PREAMBLE	; Sending preamble?
	rjmp	SOF_CHECK		; If not, check if SOF
	
	
	; Preamble
		
	sbrs	BitCountLow,0		; Check if first or second chip
	rjmp	PREAMBLE_2		; If second, jump
	
	sbi	PORTB,DIO		; Send first chip
	rjmp	PREAMBLE_3
	
PREAMBLE_2:
	cbi	PORTB,DIO		; Send second chip
	
PREAMBLE_3:
	subi	BitCountLow,1
	sbci	BitCountHigh,0		; Update the counter
	
	brne	OVF0_END		; If counter is not 0, we're done

	
TX_FINISH_PREAMBLE:
	cbr	flag_reg, 1<<PREAMBLE	; Clear PREAMBLE flag
	ldi	BitCountLow,4		; Load counter (2 bit SOF)
	rjmp	OVF0_END		; Done
	
	
SOF_CHECK:
	sbrs	flag_reg, SOF		; Sending SOF?
	rjmp	SEND_DATA		; If not, send data
	
	; SOF
	
	sbrs	BitCountLow,0		; Check if first or second chip
	rjmp	SOF_2			; If second, jump
	
	cbi	PORTB,DIO		; Send first chip
	rjmp	SOF_3		
	
SOF_2:
	sbi	PORTB,DIO		; Send second chip
SOF_3:
	dec	BitCountLow		; Update counter
	
	brne	OVF0_END		; If counter is not 0, we're done

	
TX_FINISH_SOF:	
	cbr	flag_reg, 1<<SOF	; Clear SOF flag
	ldi	BitCountLow,0x4F	; Load counter
	rjmp	OVF0_END		; Done
	

SEND_DATA:	

	sbrs	BitCountLow,0		; Check if first or second chip
	rjmp	SECOND_CHIP		; If second chip, jump
	
	; First chip
	
	; Shift data to get at next bit
	rol	data3
	rol	data2
	rol	data1
	rol	type
	rol	id
	
	; Data to be sent is now in carry
	
	brcs	CHIP1_1			; Jump if data is a 1
	
	cbr	flag_reg, 1<<LAST_BIT	
	cbi	PORTB, DIO		; Send a 0
	rjmp	DATA_SEND_END
CHIP1_1:
	sbr	flag_reg, 1<<LAST_BIT
	sbi	PORTB, DIO		; Send a 1

DATA_SEND_END:
	dec	BitCountLow		; Update counter
	brne	OVF0_END		; If not zero, done
	
	; We have sent all data, go back to polling
	
	cbr	flag_reg,1<<SEND_ACK
	rcall	START_POLLING_STATE
	rjmp	OVF0_END


	; End of interrupt
OVF0_END:
	ldi	temp, 0xFF
	out	TIFR, temp		; Clear timer interrupt	
	out	SREG, SREG_SAVE		; Restore SREG

	reti	

	; Handle transmission of second chip
SECOND_CHIP:
	sbrs	flag_reg,LAST_BIT	; Skip if data is a 1
	rjmp	CHIP2_0
	
	; Bit is a 1
	cbi	PORTB, DIO
	rjmp	DATA_SEND_END
	
CHIP2_0:
	; Bit is a 0
	sbi	PORTB, DIO
	rjmp	DATA_SEND_END


; Timer overflow handler in POLLING state

OVF0_POLLING:
	ldi	temp, POLLING_TIMER_PRESET
	out	TCNT0, temp		; Set up timer for new period

	inc	poll_count	
	mov	temp, poll_count
	andi	temp, 0x03		; Is counter divisible by 4?
	brne	OVF0_END		; If not, return immediatly

	tst	poll_count	
	breq	POLL_OVERFLOW		; If polling has gone on long enough,
					; enter power-down mode
	
	LED_ON				; Turn LED on (heartbeat)
	
	
	rcall	SETUP_CC400_RX_PC	; Configure CC400 for RX precharge
	
	rcall	START_PREAMBLE_STATE	; Ready to search for preamble
	
	; delay for CC400 precharge
	
	ldi	temp,0x00
PC_DELAY:
	ldi	ee_counter,0x20
PC_DELAY_INNER:
	subi	ee_counter,0x01
	brne	PC_DELAY_INNER
	
	subi	temp,0x01	
	brne	PC_DELAY
	
	rcall	SETUP_CC400_RX		; Precharge finish, enter RX mode
	
	LED_OFF				; Turn off LED
	
	; Set up INT0 to detect first positive edge
	
	ldi	temp,0xff
	out	GIFR,temp		; Clear interrupt flags
	
	ldi	temp,0x40
	out	GIMSK, temp		; Enable INT0
	
	ldi	temp, 0x00		
	out	TIMSK, temp		; Disable timer/counter
					; to prevent it interfering with
					; INT0 interrupt
	
	rjmp	OVF0_END
	
POLL_OVERFLOW:				; Counter overflowed
	rcall	START_PD_STATE		; Go into power down mode
	rjmp	OVF0_END
	

	ldi	temp, PREAMBLE_TIMER_PRESET
	out	TCNT0, temp		; Set up timer for new period
	rjmp	OVF0_END		; Done		


; Timer overflow handler for DISPLAY state

OVF0_DISPLAY:
	ldi	temp, DISPLAY_TIMER_PRESET
	out	TCNT0, temp		; Reload timer
	
	rol	data3
	rol	data2
	rol	data1			; Shift the bits left
	
	; Display bit in carry
	
	LED_OFF				; Turn LED off
	brcc	LED_FINISH		; Jump if LED should be off
	LED_ON				; If not, turn LED on
	
LED_FINISH:
	; LED is now lit according to data
	
	dec	BitCountLow		; Update counter
	brne	OVF0_END		; Finished if not last bit
	
	; Last bit sent, determine next mode
	
	sbrc	flag_reg, SEND_ACK	; Check if an ACK should be sent
	rjmp	DISPLAY_ACK		; If yes, jump
	
	; ACK should not be sent
	rcall	START_POLLING_STATE	; Go back to polling
	rjmp	OVF0_END
	
DISPLAY_ACK:
	rcall	START_TX_STATE		; Go into TX mode
	rjmp	OVF0_END	
	



	
;*****************************************************************************
;
; External interrupt handler
;
;*****************************************************************************

INT0_HANDLER:
	in	SREG_SAVE,SREG		; Save SREG
	
	; INT0 is used in 2 cases:
	; Either we just woke up from sleep (ON button pressed),
	; or INT0 was used to detect the rising edge when checking
	; for a preamble
	
	; Check state variable to be sure
	
	cpi	STATE, STATE_PD		; Power-down?
	breq	PD_WAKE			; If yes, handle it
	
	; Detected rising edge in preamble
	
	ldi	temp, 0xFF
	out	TCNT0, temp		; Make sure we get a timer interrupt
					; as soon as possible
	
	; Enable timer/counter
	ldi	temp, 0x02
	out	TIMSK, temp
	
	
	clr	temp
	out	GIMSK, temp		; Disable INT0
	
	rjmp	INT0_END		; Quit so timer interrupt can execute

	
PD_WAKE:				; We just woke up from sleep
	LED_ON				; Turn on LED	
	nop
	
WAIT_FOR_ON_RELEASE:
	sbis	PINB, ON_BUTTON
	rjmp	WAIT_FOR_ON_RELEASE	; Wait until on button is released
	
	LED_OFF				; Turn off LED
	
	rcall	START_POLLING_STATE	; Go into polling state	
	
INT0_END:

	out	SREG,SREG_SAVE		; Restore SREG

	reti


;*****************************************************************************
;
; Pin change interrupt handler
;
; Used to detect pushing of TX button only, ON button handled by using
; INT0. Should be disabled when we are in states other than polling, as
; a transition on any pin (even one configured as an output) trigger this
; interrupt
;
;*****************************************************************************

PCINT:
	in	SREG_SAVE,SREG		; Save SREG
	sbic 	PINB, TX_BUTTON		; If TX not pushed, exit interrupt
	rjmp	PCINT_END
	
	; TX is pushed, change state
	
	rcall	START_TX_STATE
	
	
PCINT_END:
	out	SREG,SREG_SAVE		; Restore SREG
	reti				; Return from interrupt
	
	


;*****************************************************************************
;
; START_TX_STATE routine
;
;*****************************************************************************

START_TX_STATE:
	
	rcall	SETUP_CC400_TX		; Configure CC400
	
	; Set up variables
	
	
	ldi	temp, DATA1_VALUE	; Set up data to be transmitted
	mov	data1, temp
	ldi	temp, DATA2_VALUE
	mov	data2, temp
	ldi	temp, DATA3_VALUE
	mov	data3, temp
	
	ldi	temp, ID_BYTE
	mov	id, temp
	ldi	temp, PACKET_REQUEST
	sbrc	flag_reg, SEND_ACK	; Is this an ACK?
	ldi	temp, PACKET_ACKNOWLEDGE
	mov	type, temp
	
	clr	poll_count		; TX button resets time-out
	
	sbr	flag_reg, 1<<PREAMBLE	
	sbr	flag_reg, 1<<SOF
	
	cbr	flag_reg, 1<<SEND_ACK
	
	; Load bit counter with number of bits in preamble*2 (2 intr/bit)
	ldi	BitCountLow,low(PREAMBLE_LENGTH<<1)+1
	ldi	BitCountHigh,high(PREAMBLE_LENGTH<<1)
	
	ldi	ZL, low(AVR_TX<<1)
	ldi	ZH, high(AVR_TX<<1)	
	rcall	AVR_CONFIGURE		; Configure AVR for TX state
	

	ret
	
START_POLLING_STATE:

	ldi	ZL, low(AVR_POLLING<<1)
	ldi	ZH, high(AVR_POLLING<<1)
	rcall	AVR_CONFIGURE		; Configure AVR for polling state
	
	rcall	SETUP_CC400_PD		; Configure CC400
	
	wdr				; Reset watchdog
	ldi	temp, 0x0e		; Enable watchdog, 1024 cycles
	out	WDTCR, temp

	
	ret
	
START_PREAMBLE_STATE:

	ldi	ZL, low(AVR_PREAMBLE<<1)
	ldi	ZH, high(AVR_PREAMBLE<<1)
	rcall	AVR_CONFIGURE		; Configure AVR for preamble search
	
	; Init variables
	
	ldi	An0,0x08
	mov	An1, An0
	mov	An2, An0
	cbr	flag_reg, 1<<FIRST_BIT_STORED
	clr	Sample
	clr	ShiftReg
	
	ret

	
START_SOFSEARCH:
	; CC400 is in RX from the previous state, no configuration
	; necessary
	
	clr	BitCountLow
	clr	BitCountHigh
	ldi	error_allow,MAX_SOF_ERRORS
	
	ldi	state, STATE_SOFSEARCH

	ldi	ZL, low(AVR_SOFSEARCH<<1)
	ldi	ZH, high(AVR_SOFSEARCH<<1)
	rcall	AVR_CONFIGURE		; Configure AVR for SOF search
	
	ret
	
	
	
START_PD_STATE:

	rcall	SETUP_CC400_PD		; Configure CC400
	
	rcall	SETUP_CC400_PD		; Configure again, to decrease chance
					; of bit errors leaving the CC400
					; powered up
					
	clr	poll_count		; Reset polling counter

	ldi	ZL, low(AVR_PD<<1)
	ldi	ZH, high(AVR_PD<<1)
	rcall	AVR_CONFIGURE		; Configure AVR for power-down
	
	
	wdr				; Strobe watchdog, don't want
					; an interrupt to occur now
	
	ldi	temp, 0x19		; Disable watchdog step 1
	out	WDTCR, temp		
	
	ldi	temp, 0x01		; Disable watchdog step 2
	out	WDTCR, temp
	
	ret
	
START_RX_STATE:

	clr	BitCountLow
	clr	BitCountHigh
	clr	data_buffer
	
; No configuration of CC400 necessary, device is already in RX as the RX
; state can only be entered from the SOF search state
	
	ldi	state, STATE_RX
	ldi	ZL, low(AVR_RX<<1)
	ldi	ZH, high(AVR_RX<<1)
	rcall	AVR_CONFIGURE		; Configure AVR for RX
	
	ret
	
START_DISPLAY_STATE:

	ldi	BitCountLow, 0x18	; Preset counter
	
	ldi	ZL, low(AVR_DISPLAY<<1)
	ldi	ZH, high(AVR_DISPLAY<<1)
	rcall	AVR_CONFIGURE		; Configre AVR for display
	
	rcall	SETUP_CC400_PD		; Configure CC400
	
	ret
	
	
	
;*****************************************************************************
;
; Routine:	AVR_CONFIGURE
;
; Configures AVR peripherals according to current state
;
; Parameters:	Z (R30:R31) set to address of first parameter (program memory)
;
; Registers affected : R0, R30,R31
;
;*****************************************************************************

AVR_CONFIGURE:
	
	lpm				; Load state value
	mov	state,R0
	
	add	ZL, OneReg
	adc	ZH, ZeroReg
	lpm				; Load PORTB value
	out	PORTB,R0
	
	add	ZL, OneReg
	adc	ZH, ZeroReg
	lpm				; Load DDRB value
	out	DDRB,R0
	
	add	ZL, OneReg
	adc	ZH, ZeroReg
	lpm				; Load MCUCR value
	out	MCUCR,R0
	
	add	ZL, OneReg
	adc	ZH, ZeroReg
	lpm				; Load GIMSK value
	out	GIMSK,R0
	
	add	ZL, OneReg
	adc	ZH, ZeroReg
	lpm				; Load timer prescale value
	out	TCCR0,R0
	
	add	ZL, OneReg
	adc	ZH, ZeroReg
	lpm				; Load timer preset value
	out	TCNT0,R0
	
	ret

	
	

;*****************************************************************************
;
; Routine:	SETUP_CC400_TX
;
; Configure CC400 for TX
;
;*****************************************************************************

SETUP_CC400_TX:
	ldi	ee_counter, 8		; Load 8 words
	ldi	ee_pointer, TX		; Point to TX data
	rjmp	CC400_CONFIGURE		; Go to configuration routine
	
	
;*****************************************************************************
;
; Routine:	SETUP_CC400_RX_PC
;
; Configure CC400 for RX precharge
;
;*****************************************************************************

SETUP_CC400_RX_PC:
	ldi	ee_counter,8		; Load 8 words
	ldi	ee_pointer, RX_PC	; Point to RX_PC data
	rjmp	CC400_CONFIGURE		; Go to configuration routine
	
;*****************************************************************************
;
; Routine:	SETUP_CC400_RX
;
; Configure CC400 for RX after RX precharge has been active
;
;*****************************************************************************

SETUP_CC400_RX:
	ldi	ee_counter,1		; Load 1 word
	ldi	ee_pointer, RX		; Point to RX data
	rjmp	CC400_CONFIGURE		; Go to configuration routine
	
	
;*****************************************************************************
;
; Routine:	SETUP_CC400_PD
;
; Configure CC400 for Power down
;
;*****************************************************************************

SETUP_CC400_PD:
	ldi	ee_counter,2		; Load 2 words
	ldi	ee_pointer, PD		; Point to PD data
	rjmp	CC400_CONFIGURE		; Go to configuration routine
	
;*****************************************************************************
;
; Routine:	CC400_CONFIGURE
;
; Configures the CC400 with data from EEPROM
;
; Input parameters :
;	ee_counter	: Number of words to load into CC400
;	ee_pointer	: Pointer to data to be loaded
;
;
;
;*****************************************************************************

CC400_CONFIGURE:

CC400_WORD:
	cbi	PORTB, STROBE		; Set STROBE to 0
	sbi	DDRB, STROBE		; Strobe is output
	cbi	PORTB, CLOCK		; Set CLOCK to 0
	sbi	DDRB, CLOCK		; Set CLOCK to output
	
	sbi	DDRB, PDATA		; Set data as output
	
EE_GET1:
	sbic	EECR, EEWE		; EEPROM ready?
	rjmp	EE_GET1			; If not, wait
	
	out	EEAR, ee_pointer	; Set EEPROM address
	sbi	EECR, EERE		; Do EEPROM read
	
	in	ee_buffer, EEDR		; Read data
	
	ldi	temp,8			; Set bit counter
	
CC400_BYTE_LOOP1:
	
	; Set PDATA equal to current bit
	sbrc	ee_buffer,7
	sbi	PORTB, PDATA
	sbrs	ee_buffer,7
	cbi	PORTB, PDATA
	
	lsl	ee_buffer		; Select next bit
	
	; Send clock pulse
	sbi	PORTB, CLOCK
	nop
	nop
	nop
	nop
	nop
	cbi	PORTB, CLOCK
	
	dec	temp			; Are we finished with all bits?
	brne	CC400_BYTE_LOOP1	; If no, loop again
	
	
	inc	ee_pointer		; Next byte

EE_GET2:
	sbic	EECR, EEWE		; EEPROM ready?
	rjmp	EE_GET2			; If not, wait
	
	out	EEAR, ee_pointer	; Set EEPROM address
	sbi	EECR, EERE		; Do EEPROM read
	
	in	ee_buffer, EEDR		; Read data
	
	ldi	temp,8			; Set bit counter
	
CC400_BYTE_LOOP2:
	
	; Set PDATA equal to current bit
	sbrc	ee_buffer,7
	sbi	PORTB, PDATA
	sbrs	ee_buffer,7
	cbi	PORTB, PDATA
	
	lsl	ee_buffer		; Select next bit
	
	; Send clock pulse
	sbi	PORTB, CLOCK
	nop
	cbi	PORTB, CLOCK
	
	dec	temp			; Are we finished with all bits?
	brne	CC400_BYTE_LOOP2	; If no, loop again
	
	
	; Send strobe
	
	sbi	PORTB, STROBE
	nop
	nop
	nop
	cbi	PORTB, STROBE
	
	
	
	inc	ee_pointer		; Set pointer to next byte
	dec	ee_counter		; Have we sent all the words required?
	brne	CC400_WORD		; If not, send next word
	
	sbi	PORTB, CLOCK		; Set clock to 1 (need pullup)
	
	
	cbi	DDRB, CLOCK		; Set CLOCK to input again	
	ldi	temp, 0xFF
	out	GIFR, temp		; Clear any interrupt flags
	ret				; Return to caller


.cseg

; AVR parameters for each state, 8 bytes for each state

.ORG	0x01E0

AVR_POLLING:
	.db	STATE_POLLING, PORTB_POLLING, DDRB_POLLING, MCUCR_POLLING, GIMSK_POLLING, POLLING_TIMER_PRESCALE, POLLING_TIMER_PRESET, 0
AVR_TX:
	.db	STATE_TX, PORTB_TX, DDRB_TX, MCUCR_TX, GIMSK_TX, TX_TIMER_PRESCALE, TX_TIMER_PRESET, 0
AVR_RX:
	.db	STATE_RX, PORTB_RX, DDRB_RX, MCUCR_RX, GIMSK_RX, RX_TIMER_PRESCALE, RX_TIMER_PRESET, 0
AVR_PD:
	.db	STATE_PD, PORTB_PD, DDRB_PD, MCUCR_PD, GIMSK_PD, PD_TIMER_PRESCALE, PD_TIMER_PRESET, 0
AVR_PREAMBLE:
	.db	STATE_PREAMBLE, PORTB_PREAMBLE, DDRB_PREAMBLE, MCUCR_PREAMBLE, GIMSK_PREAMBLE, PREAMBLE_TIMER_PRESCALE, 0x00, 0
AVR_SOFSEARCH:
	.db	STATE_SOFSEARCH, PORTB_SOFSEARCH, DDRB_SOFSEARCH, MCUCR_SOFSEARCH, GIMSK_SOFSEARCH, SOFSEARCH_TIMER_PRESCALE, SOFSEARCH_TIMER_PRESET, 0
AVR_DISPLAY:
	.db	STATE_DISPLAY, PORTB_DISPLAY, DDRB_DISPLAY, MCUCR_DISPLAY, GIMSK_DISPLAY, DISPLAY_TIMER_PRESCALE, DISPLAY_TIMER_PRESET, 0



	
.ORG 0x1FF	; Last word in FLASH

CALIBRATION_BYTE:
	.db 0xFF,0xFF		; Load the calibration byte here